package net.knarcraft.ffmpegconverter.converter;

import net.knarcraft.ffmpegconverter.streams.StreamObject;
import net.knarcraft.ffmpegconverter.streams.VideoStream;
import net.knarcraft.ffmpegconverter.utility.FFMpegHelper;

import java.io.File;
import java.util.List;

/**
 * A converter for converting video files
 */
public class DownScaleConverter extends AbstractConverter {

    private final int newWidth;
    private final int newHeight;

    /**
     * Instantiates a new video converter
     *
     * @param ffprobePath <p>Path/command to ffprobe.</p>
     * @param ffmpegPath  <p>Path/command to ffmpeg.</p>
     * @param newWidth    <p>The new width of the video</p>
     * @param newHeight   <p>The new height of the video</p>
     */
    public DownScaleConverter(String ffprobePath, String ffmpegPath, int newWidth, int newHeight) {
        super(null);
        this.ffprobePath = ffprobePath;
        this.ffmpegPath = ffmpegPath;
        this.newHeight = newHeight;
        this.newWidth = newWidth;
    }

    @Override
    public String[] generateConversionCommand(String executable, File file, List<StreamObject> streams, String outFile) {
        VideoStream videoStream = getNthVideoStream(streams, 0);
        if (videoStream == null || (videoStream.getWidth() <= newWidth && videoStream.getHeight() <= newHeight)) {
            return new String[0];
        }

        List<String> command = FFMpegHelper.getFFMpegGeneralFileCommand(executable, file.getName());
        if (this.debug) {
            FFMpegHelper.addDebugArguments(command, 50, 120);
        }

        //Add all streams without re-encoding
        command.add("-map");
        command.add("0");
        command.add("-c:a");
        command.add("copy");
        command.add("-c:s");
        command.add("copy");
        command.add("-vf");
        command.add("scale=" + newWidth + ":" + newHeight);
        command.add("-crf");
        command.add("20");
        command.add("-preset");
        command.add("slow");

        command.add(outFile);
        return command.toArray(new String[0]);
    }

    @Override
    public String[] getValidFormats() {
        return videoFormats;
    }

}